@@ -26,10 +26,11 @@ recluster.DSC <- function(macro, dsc, type="auto", ...) {
 		    paste(class(macro), collapse=", ")))
 }
 
+### reclustering is done with a DSC_Macro object!
 recluster.DSC_Macro <- function(macro, dsc, type="auto", ...) {
-    
-    x <- get_centers(dsc, type=type)
+    cen <- get_centers(dsc, type=type)
+    dsd <- DSD_Wrapper(cen)
     weight <- get_weights(dsc, scale=NULL, type=type)
-    macro$RObj$cluster(x, weight=weight, ...)
+    cluster(macro, dsd, n=nrow(cen), weight=weight, ...)
 }
 
